package bzr4idea;

import com.intellij.openapi.vcs.VcsException;
import com.intellij.openapi.vcs.annotate.AnnotationProvider;
import com.intellij.openapi.vcs.annotate.FileAnnotation;
import com.intellij.openapi.vcs.history.VcsFileRevision;
import com.intellij.openapi.vfs.VirtualFile;
import org.vcs.bazaar.client.IBazaarAnnotation;
import org.vcs.bazaar.client.commandline.syntax.IAnnotateOptions;
import org.vcs.bazaar.client.core.BazaarClientException;

import java.io.File;

public class BzrAnnotationProvider implements AnnotationProvider {

	private BzrVcs bzr;

	public BzrAnnotationProvider(BzrVcs bzr) {

		this.bzr = bzr;
	}

	public FileAnnotation annotate(VirtualFile file) throws VcsException {
		return annotate(file, null);
	}

	public FileAnnotation annotate(VirtualFile file, VcsFileRevision revision) throws VcsException {

		try {
			IBazaarAnnotation annotation = bzr.getBzrClient().annotate(
					new File(file.getPath()),
					IAnnotateOptions.REVISION.setArgument(revision.getRevisionNumber().asString()));

			return new BzrFileAnnotation(annotation);

		} catch (BazaarClientException e) {
			throw Assert.notYetHandled(e);
		}
	}

	public boolean isAnnotationValid(VcsFileRevision rev) {
		return true;
	}
}
