package bzr4idea;

import static bzr4idea.Utils.root;
import com.intellij.openapi.vcs.CheckinProjectPanel;
import com.intellij.openapi.vcs.FilePath;
import com.intellij.openapi.vcs.VcsException;
import com.intellij.openapi.vcs.changes.Change;
import com.intellij.openapi.vcs.changes.ContentRevision;
import com.intellij.openapi.vcs.checkin.CheckinEnvironment;
import com.intellij.openapi.vcs.ui.RefreshableOnComponent;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.vcsUtil.VcsUtil;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.Nullable;
import org.vcs.bazaar.client.utils.BazaarUtilities;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class BzrCheckinEnvironment implements CheckinEnvironment {
    private BzrVcs bzr;

    public BzrCheckinEnvironment(BzrVcs bzrVcs) {
        this.bzr = bzrVcs;
    }

    @Nullable
    public RefreshableOnComponent createAdditionalOptionsPanel(CheckinProjectPanel panel) {
        return null; // todo: create "local commit" option panel
    }

    @Nullable
    public String getDefaultMessageFor(FilePath[] filesToCheckin) {
        return null;
    }

    public String prepareCheckinMessage(String text) {
        return text;
    }

    @Nullable
    @NonNls
    public String getHelpId() {
        return null;
    }

    public String getCheckinOperationName() {
        return "Commit";
    }

    public boolean showCheckinDialogInAnyCase() {
        return false;
    }

    @Nullable
    public List<VcsException> commit(List<Change> changes, String preparedComment) {
		 Set<File> roots = getBzrRoots(changes);

		 if (roots.size() > 1) {
			 return Arrays.asList(new VcsException(String.format(
					 "Too many VCS roots. Cannot commit to multiple VCS roots at once. Roots (%s): %s",
					 roots.size(), roots)));
		 }

		 List<File> files = new ArrayList<File>(changes.size());
		  for(Change change : changes) {
			  ContentRevision revision = change.getAfterRevision();
			  if (revision == null) {
				  revision = change.getBeforeRevision();
			  }

			  files.add(revision.getFile().getIOFile());
        }

        try {
			  bzr.getBzrClient().setWorkDir(root(files.iterator().next()));
			  bzr.getBzrClient().commit(files.toArray(new File[files.size()]), preparedComment);
			  bzr.onSuccessfulCommit(files);

			  return null;
		  } catch (Exception e) {
            return Arrays.asList(new VcsException(e));
        }
    }

	@Nullable
    public List<VcsException> scheduleMissingFileForDeletion(List<FilePath> files) {
        throw new UnsupportedOperationException("scheduleMissingFileForDeletion: " + files);
    }

    @Nullable
    public List<VcsException> scheduleUnversionedFilesForAddition(List<VirtualFile> files) {
        throw new UnsupportedOperationException("scheduleUnversionedFilesForAddition: " + files);
    }

	private Set<File> getBzrRoots(List<Change> changes) {
		// 1) get roots as suggested by IDEA
		Set<VirtualFile> vroots = new HashSet<VirtualFile>();
		for (Change change : changes) {
			ContentRevision rev = change.getAfterRevision() != null ? change.getAfterRevision() : change.getBeforeRevision();
			FilePath f = rev.getFile();
			VirtualFile root = VcsUtil.getVcsRootFor(bzr.getProject(), f);
			vroots.add(root);
		}

		// 2) normalize for bzr workspace: ask bzr about all IDEA VCS roots
		Set<File> roots = new HashSet<File>();
		for (VirtualFile vfile : vroots) {
			File root = BazaarUtilities.getRootBranch(new File(vfile.getPath()));
			roots.add(root);
		}
		return roots;
	}


}
