package bzr4idea;

import com.intellij.openapi.vcs.ChangeListColumn;
import com.intellij.openapi.vcs.CommittedChangesProvider;
import com.intellij.openapi.vcs.DefaultRepositoryLocation;
import com.intellij.openapi.vcs.FilePath;
import com.intellij.openapi.vcs.RepositoryLocation;
import com.intellij.openapi.vcs.VcsException;
import com.intellij.openapi.vcs.versionBrowser.ChangeBrowserSettings;
import com.intellij.openapi.vcs.versionBrowser.CommittedChangeList;
import com.intellij.openapi.vcs.versionBrowser.CommittedChangeListImpl;
import com.intellij.openapi.vcs.versionBrowser.StandardVersionFilterComponent;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.client.utils.BazaarUtilities;

import javax.swing.*;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

public class BzrCommittedChangesProvider implements CommittedChangesProvider<CommittedChangeList, ChangeBrowserSettings> {
	private BzrVcs bzr;

	public BzrCommittedChangesProvider(BzrVcs bzrVcs) {
		this.bzr = bzrVcs;
	}

	public ChangeBrowserSettings createDefaultSettings() {
		return new ChangeBrowserSettings();
	}

	public StandardVersionFilterComponent<ChangeBrowserSettings> createFilterUI(boolean showDateFilter) {
		return new StandardVersionFilterComponent<ChangeBrowserSettings>(showDateFilter) {
			public JComponent getComponent() {
				return new JPanel();
			}
		};
	}

	public RepositoryLocation getLocationFor(FilePath root) {
		try {
			return new DefaultRepositoryLocation(
					BazaarUtilities.getRootBranch(new File(root.getPath())).getPath());
		} catch (Exception e) {
			throw Assert.notYetHandled(e);
		}
	}

	public List<CommittedChangeList> getCommittedChanges(ChangeBrowserSettings settings,
																		  RepositoryLocation location,
																		  int maxCount) throws VcsException {
		try {
			File root = new File(location.toPresentableString());
			bzr.getBzrClient().setWorkDir(root);
			IBazaarLogMessage[] messages = bzr.getBzrClient().log(root);

			List<CommittedChangeList> revisions = new ArrayList<CommittedChangeList>(messages.length);
			for (IBazaarLogMessage msg : messages) {
				revisions.add(new CommittedChangeListImpl("name?", msg.getMessage(), msg.getCommiter(),
																		0, msg.getDate(), null
				));
			}

			return revisions;

		} catch (BazaarClientException e) {
			throw Assert.notYetHandled(e);
		}
	}

	public ChangeListColumn[] getColumns() {
		return new ChangeListColumn[]{ChangeListColumn.DATE, ChangeListColumn.DESCRIPTION, ChangeListColumn.NAME, ChangeListColumn.NUMBER};
	}
}
