package bzr4idea;

import static bzr4idea.Utils.root;
import com.intellij.openapi.util.io.FileUtil;
import com.intellij.openapi.vcs.FilePath;
import com.intellij.openapi.vcs.VcsException;
import com.intellij.openapi.vcs.changes.ContentRevision;
import com.intellij.vcsUtil.VcsUtil;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.vcs.bazaar.client.core.BazaarClientException;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;

public class BzrContentRevision implements ContentRevision {

	private BzrVcs bzr;
	private File file;
	private String revision;
	private String content;

	public BzrContentRevision(BzrVcs bzr, File file, String revision) {
		this.bzr = bzr;
		this.file = file;
		this.revision = revision;
	}

	@Nullable
	public String getContent() throws VcsException {
		if (content != null) {
			return content;
		}

		try {
			bzr.getBzrClient().setWorkDir(root(file));
			if (file.isFile()) {
				InputStream stream = bzr.getBzrClient().cat(file, getRevisionNumber().getBazaarRevision());
				content = new String(FileUtil.adaptiveLoadBytes(stream));
			}

			return content;

		} catch (BazaarClientException e) {
			throw Assert.notYetHandled(e);
		} catch (IOException e) {
			throw Assert.notYetHandled(e);
		}

	}

	@NotNull
	public FilePath getFile() {
		return VcsUtil.getFilePath(file);
	}

	@NotNull
	public BzrRevisionNumber getRevisionNumber() {
		return bzr.parseRevisionNumber(revision);
	}
}
