package bzr4idea;

import static bzr4idea.Utils.root;
import com.intellij.openapi.util.io.FileUtil;
import com.intellij.openapi.vcs.VcsException;
import com.intellij.openapi.vcs.history.VcsFileRevision;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.client.core.BazaarClientException;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Date;

public class BzrFileRevision implements VcsFileRevision {

	private BzrVcs bzr;
	private File file;
	private IBazaarLogMessage message;


	public BzrFileRevision(BzrVcs bzr, File file, IBazaarLogMessage message) {
		this.bzr = bzr;

		this.file = file;
		this.message = message;
	}

	public BzrRevisionNumber getRevisionNumber() {
        return new BzrRevisionNumber(message.getRevision());
    }

    public String getBranchName() {
        return message.getBranchNick();
    }

    public Date getRevisionDate() {
        return message.getDate();
    }

    public String getAuthor() {
        return message.getCommiter();
    }

    public String getCommitMessage() {
        return message.getMessage();
    }

    public void loadContent() throws VcsException {
    }

    public byte[] getContent() throws IOException {

		 try {
			 bzr.getBzrClient().setWorkDir(root(file));
			 InputStream stream = bzr.getBzrClient().cat(file, message.getRevision());
			 return FileUtil.adaptiveLoadBytes(stream);

		 } catch (BazaarClientException e) {
			 throw Assert.notYetHandled(e);
		 }
    }
}
