package bzr4idea;

import static bzr4idea.Utils.root;
import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.vcs.FilePath;
import com.intellij.openapi.vcs.VcsException;
import com.intellij.openapi.vcs.history.FileHistoryPanel;
import com.intellij.openapi.vcs.history.HistoryAsTreeProvider;
import com.intellij.openapi.vcs.history.VcsFileRevision;
import com.intellij.openapi.vcs.history.VcsHistoryProvider;
import com.intellij.openapi.vcs.history.VcsHistorySession;
import com.intellij.util.ui.ColumnInfo;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.Nullable;
import org.vcs.bazaar.client.BazaarRevision;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.client.core.BazaarClientException;

import java.util.ArrayList;
import java.util.List;
import java.io.File;

public class BzrHistoryProvider implements VcsHistoryProvider {

	private BzrVcs bzr;

	public BzrHistoryProvider(BzrVcs bzr) {
		this.bzr = bzr;
	}

	public ColumnInfo[] getRevisionColumns() {
		return new ColumnInfo[0];
	}

	public AnAction[] getAdditionalActions(FileHistoryPanel panel) {
		return new AnAction[0];
	}

	public boolean isDateOmittable() {
		return false;
	}

	@Nullable
	@NonNls
	public String getHelpId() {
		return null;
	}

	@Nullable
	public VcsHistorySession createSessionFor(FilePath filePath) throws VcsException {
		try {
			File file = filePath.getIOFile();

			BazaarRevision revno = bzr.getBzrClient().revno(file);

			bzr.getBzrClient().setWorkDir(root(file));
			IBazaarLogMessage[] messages = bzr.getBzrClient().log(file);

			List<VcsFileRevision> revisions = new ArrayList<VcsFileRevision>();
			for (IBazaarLogMessage message : messages) {
				revisions.add(new BzrFileRevision(bzr, file, message));
			}

			return new BzrHistorySession(revisions, new BzrRevisionNumber(revno));

		} catch (BazaarClientException e) {
			throw Assert.notYetHandled(e);
		}
	}

	@Nullable
	public HistoryAsTreeProvider getTreeHistoryProvider() {
		// todo: try to implement this feature cause bzr has tree history
		return null;
	}
}
