package bzr4idea;

import com.intellij.openapi.vcs.FilePath;
import com.intellij.openapi.vcs.VcsException;
import com.intellij.openapi.vcs.changes.Change;
import com.intellij.openapi.vcs.changes.ContentRevision;
import com.intellij.openapi.vcs.rollback.DefaultRollbackEnvironment;
import org.vcs.bazaar.client.core.BazaarClientException;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

public class BzrRollbackEnvironment extends DefaultRollbackEnvironment {
	
	private BzrVcs bzr;

	public BzrRollbackEnvironment(BzrVcs bzr) {
		this.bzr = bzr;
	}

	public List<VcsException> rollbackChanges(List<Change> changes) {
		try {
			List<File> files = new ArrayList<File>();

			for (Change change : changes) {
				ContentRevision rev = (change.getAfterRevision() != null)
						? change.getAfterRevision()
						: change.getBeforeRevision();
				String path = rev.getFile().getPath();
				files.add(new File(path));
			}
			bzr.getBzrClient().revert(files.toArray(new File[files.size()]));
			return null;

		} catch (BazaarClientException e) {
			throw Assert.notYetHandled(e);
		}
	}

	public List<VcsException> rollbackMissingFileDeletion(List<FilePath> files) {
		throw new UnsupportedOperationException("rollbackMissingFileDeletion: " + files);
	}
}
